@@ -16,13 +16,12 @@ import jinja2
 import yaml
 import re
 import os
-from typing import (Any, DefaultDict, Dict, Mapping)
+from typing import Any, DefaultDict, Dict, Mapping
 from hashlib import sha256
-from collections import (OrderedDict, defaultdict)
-from gapic.samplegen_utils.utils import (
-    coerce_response_name, is_valid_sample_cfg)
+from collections import OrderedDict, defaultdict
+from gapic.samplegen_utils.utils import coerce_response_name, is_valid_sample_cfg
 from gapic.samplegen_utils.types import DuplicateSample
-from gapic.samplegen import (manifest, samplegen)
+from gapic.samplegen import manifest, samplegen
 from gapic.generator import options
 from gapic.generator import formatter
 from gapic.schema import api
@@ -45,7 +44,7 @@ class Generator:
     """
 
     def __init__(self, opts: options.Options) -> None:
-        # Create the jinja environment with which to render templates.
+      # Create the jinja environment with which to render templates.
         self._env = jinja2.Environment(
             loader=jinja2.FileSystemLoader(searchpath=opts.templates),
             undefined=jinja2.StrictUndefined,
@@ -53,18 +52,16 @@ class Generator:
         )
 
         # Add filters which templates require.
-        self._env.filters['rst'] = utils.rst
-        self._env.filters['snake_case'] = utils.to_snake_case
-        self._env.filters['sort_lines'] = utils.sort_lines
-        self._env.filters['wrap'] = utils.wrap
-        self._env.filters['coerce_response_name'] = coerce_response_name
+        self._env.filters["rst"] = utils.rst
+        self._env.filters["snake_case"] = utils.to_snake_case
+        self._env.filters["sort_lines"] = utils.sort_lines
+        self._env.filters["wrap"] = utils.wrap
+        self._env.filters["coerce_response_name"] = coerce_response_name
 
         self._sample_configs = opts.sample_configs
 
     def get_response(
-        self,
-        api_schema: api.API,
-        opts: options.Options
+        self, api_schema: api.API, opts: options.Options
     ) -> CodeGeneratorResponse:
         """Return a :class:`~.CodeGeneratorResponse` for this library.
 
@@ -80,11 +77,11 @@ class Generator:
             files and contents. See ``plugin.proto``.
         """
         output_files: Dict[str, CodeGeneratorResponse.File] = OrderedDict()
-
         sample_templates, client_templates = utils.partition(
             lambda fname: os.path.basename(
                 fname) == samplegen.DEFAULT_TEMPLATE_NAME,
-            self._env.loader.list_templates())
+            self._env.loader.list_templates(),
+        )
 
         # Iterate over each template and add the appropriate output files
         # based on that template.
@@ -93,31 +90,30 @@ class Generator:
         # and plug those into the template.
         for template_name in client_templates:
             # Sanity check: Skip "private" templates.
-            filename = template_name.split('/')[-1]
-            if filename.startswith('_') and filename != '__init__.py.j2':
+            filename = template_name.split("/")[-1]
+            if filename.startswith("_") and filename != "__init__.py.j2":
                 continue
 
             # Append to the output files dictionary.
             output_files.update(
                 self._render_template(
-                    template_name,
-                    api_schema=api_schema,
-                    opts=opts
-                )
+                    template_name, api_schema=api_schema, opts=opts)
             )
 
-        output_files.update(self._generate_samples_and_manifest(
-            api_schema,
-            self._env.get_template(sample_templates[0]),
-        ))
+        output_files.update(
+            self._generate_samples_and_manifest(
+                api_schema, self._env.get_template(sample_templates[0]),
+            )
+        )
 
         # Return the CodeGeneratorResponse output.
-        return CodeGeneratorResponse(file=[i for i in output_files.values()])
+        res = CodeGeneratorResponse(
+            file=[i for i in output_files.values()])  # type: ignore
+        res.supported_features |= CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL  # type: ignore
+        return res
 
     def _generate_samples_and_manifest(
-            self,
-            api_schema: api.API,
-            sample_template: jinja2.Template,
+        self, api_schema: api.API, sample_template: jinja2.Template,
     ) -> Dict[str, CodeGeneratorResponse.File]:
         """Generate samples and samplegen manifest for the API.
 
@@ -130,8 +126,8 @@ class Generator:
         # The two-layer data structure lets us do two things:
         # * detect duplicate samples, which is an error
         # * detect distinct samples with the same ID, which are disambiguated
-        id_to_hash_to_spec: DefaultDict[str, Dict[str, Any]] = defaultdict(
-            dict)
+        id_to_hash_to_spec: DefaultDict[str,
+                                        Dict[str, Any]] = defaultdict(dict)
 
         STANDALONE_TYPE = "standalone"
         for config_fpath in self._sample_configs:
@@ -140,7 +136,8 @@ class Generator:
 
             spec_generator = (
                 spec
-                for cfg in configs if is_valid_sample_cfg(cfg)
+                for cfg in configs
+                if is_valid_sample_cfg(cfg)
                 for spec in cfg.get("samples", [])
                 # If unspecified, assume a sample config describes a standalone.
                 # If sample_types are specified, standalone samples must be
@@ -156,10 +153,9 @@ class Generator:
                 #
                 # Ideally the sample author should pick a descriptive, unique ID,
                 # but this may be impractical and can be error-prone.
-                spec_hash = sha256(str(spec).encode('utf8')).hexdigest()[:8]
-                sample_id = (spec.get("id")
-                             or spec.get("region_tag")
-                             or spec_hash)
+                spec_hash = sha256(str(spec).encode("utf8")).hexdigest()[:8]
+                sample_id = spec.get("id") or spec.get(
+                    "region_tag") or spec_hash
                 spec["id"] = sample_id
 
                 hash_to_spec = id_to_hash_to_spec[sample_id]
@@ -180,19 +176,17 @@ class Generator:
                     spec["id"] += f"_{spec_hash}"
 
                 sample = samplegen.generate_sample(
-                    spec,
-                    api_schema,
-                    sample_template,
-                )
+                    spec, api_schema, sample_template,)
 
                 fpath = spec["id"] + ".py"
-                fpath_to_spec_and_rendered[os.path.join(out_dir, fpath)] = (spec,
-                                                                            sample)
+                fpath_to_spec_and_rendered[os.path.join(out_dir, fpath)] = (
+                    spec,
+                    sample,
+                )
 
         output_files = {
             fname: CodeGeneratorResponse.File(
-                content=formatter.fix_whitespace(sample),
-                name=fname
+                content=formatter.fix_whitespace(sample), name=fname
             )
             for fname, (_, sample) in fpath_to_spec_and_rendered.items()
         }
@@ -200,24 +194,22 @@ class Generator:
         # Only generate a manifest if we generated samples.
         if output_files:
             manifest_fname, manifest_doc = manifest.generate(
-                ((fname, spec)
-                 for fname, (spec, _) in fpath_to_spec_and_rendered.items()),
-                api_schema
+                (
+                    (fname, spec)
+                    for fname, (spec, _) in fpath_to_spec_and_rendered.items()
+                ),
+                api_schema,
             )
 
             manifest_fname = os.path.join(out_dir, manifest_fname)
             output_files[manifest_fname] = CodeGeneratorResponse.File(
-                content=manifest_doc.render(),
-                name=manifest_fname
+                content=manifest_doc.render(), name=manifest_fname
             )
 
         return output_files
 
     def _render_template(
-            self,
-            template_name: str, *,
-            api_schema: api.API,
-            opts: options.Options,
+            self, template_name: str, *, api_schema: api.API, opts: options.Options,
     ) -> Dict[str, CodeGeneratorResponse.File]:
         """Render the requested templates.
 
@@ -239,82 +231,95 @@ class Generator:
         # Sanity check: Rendering per service and per proto would be a
         # combinatorial explosion and is almost certainly not what anyone
         # ever wants. Error colorfully on it.
-        if '%service' in template_name and '%proto' in template_name:
-            raise ValueError('Template files may live under a %proto or '
-                             '%service directory, but not both.')
+        if "%service" in template_name and "%proto" in template_name:
+            raise ValueError(
+                "Template files may live under a %proto or "
+                "%service directory, but not both."
+            )
 
         # If this template should be rendered for subpackages, process it
         # for all subpackages and set the strict flag (restricting what
         # services and protos we pull from for the remainder of the method).
-        if '%sub' in template_name:
+        if "%sub" in template_name:
             for subpackage in api_schema.subpackages.values():
-                answer.update(self._render_template(template_name,
-                                                    api_schema=subpackage,
-                                                    opts=opts
-                                                    ))
-            skip_subpackages = True
+                answer.update(
+                    self._render_template(
+                        template_name, api_schema=subpackage, opts=opts
+                    )
+                )
+                skip_subpackages = True
 
         # If this template should be rendered once per proto, iterate over
         # all protos to be rendered
-        if '%proto' in template_name:
+        if "%proto" in template_name:
             for proto in api_schema.protos.values():
-                if (skip_subpackages and proto.meta.address.subpackage !=
-                        api_schema.subpackage_view):
+                if (
+                        skip_subpackages
+                        and proto.meta.address.subpackage != api_schema.subpackage_view
+                ):
                     continue
-                answer.update(self._get_file(template_name,
-                                             api_schema=api_schema,
-                                             proto=proto,
-                                             opts=opts
-                                             ))
+
+                answer.update(
+                    self._get_file(
+                        template_name, api_schema=api_schema, proto=proto, opts=opts
+                    )
+                )
+
             return answer
 
         # If this template should be rendered once per service, iterate
         # over all services to be rendered.
-        if '%service' in template_name:
+        if "%service" in template_name:
             for service in api_schema.services.values():
-                if (skip_subpackages and service.meta.address.subpackage !=
-                        api_schema.subpackage_view):
+                if (
+                        skip_subpackages
+                        and service.meta.address.subpackage != api_schema.subpackage_view
+                ):
                     continue
-                answer.update(self._get_file(template_name,
-                                             api_schema=api_schema,
-                                             service=service,
-                                             opts=opts,
-                                             ))
+
+                answer.update(
+                    self._get_file(
+                        template_name,
+                        api_schema=api_schema,
+                        service=service,
+                        opts=opts,
+                    )
+                )
             return answer
 
         # This file is not iterating over anything else; return back
         # the one applicable file.
-        answer.update(
-            self._get_file(template_name, api_schema=api_schema, opts=opts)
-        )
+        answer.update(self._get_file(
+            template_name, api_schema=api_schema, opts=opts))
         return answer
 
-    def _get_file(self, template_name: str, *,
-                  opts: options.Options,
-                  api_schema=api.API,
-                  **context: Mapping):
+    def _get_file(
+        self,
+        template_name: str,
+        *,
+        opts: options.Options,
+        api_schema=api.API,
+        **context: Mapping,
+    ):
         """Render a template to a protobuf plugin File object."""
         # Determine the target filename.
-        fn = self._get_filename(template_name,
-                                api_schema=api_schema,
-                                context=context,
-                                )
+        fn = self._get_filename(
+            template_name, api_schema=api_schema, context=context,)
 
         # Render the file contents.
         cgr_file = CodeGeneratorResponse.File(
             content=formatter.fix_whitespace(
                 self._env.get_template(template_name).render(
-                    api=api_schema,
-                    opts=opts,
-                    **context
+                    api=api_schema, opts=opts, **context
                 ),
             ),
             name=fn,
         )
 
         # Sanity check: Do not render empty files.
-        if (utils.empty(cgr_file.content) and
-                not fn.endswith(('py.typed', '__init__.py'))):
+        if utils.empty(cgr_file.content) and not fn.endswith(
+            ("py.typed", "__init__.py")
+        ):
             return {}
 
         # Return the filename and content in a length-1 dictionary
@@ -322,10 +327,7 @@ class Generator:
         return {fn: cgr_file}
 
     def _get_filename(
-            self,
-            template_name: str, *,
-            api_schema: api.API,
-            context: dict = None,
+        self, template_name: str, *, api_schema: api.API, context: dict = None,
     ) -> str:
         """Return the appropriate output filename for this template.
 
@@ -347,45 +349,40 @@ class Generator:
         Returns:
             str: The appropriate output filename.
         """
-        filename = template_name[:-len('.j2')]
+        filename = template_name[: -len(".j2")]
 
         # Replace the %namespace variable.
         filename = filename.replace(
-            '%namespace',
+            "%namespace",
             os.path.sep.join([i.lower() for i in api_schema.naming.namespace]),
         ).lstrip(os.path.sep)
 
         # Replace the %name, %version, and %sub variables.
-        filename = filename.replace('%name_%version',
-                                    api_schema.naming.versioned_module_name)
-        filename = filename.replace('%version', api_schema.naming.version)
-        filename = filename.replace('%name', api_schema.naming.module_name)
-        filename = filename.replace('%sub',
-                                    '/'.join(api_schema.subpackage_view))
+        filename = filename.replace(
+            "%name_%version", api_schema.naming.versioned_module_name
+        )
+        filename = filename.replace("%version", api_schema.naming.version)
+        filename = filename.replace("%name", api_schema.naming.module_name)
+        filename = filename.replace(
+            "%sub", "/".join(api_schema.subpackage_view))
 
         # Replace the %service variable if applicable.
-        if context and 'service' in context:
+        if context and "service" in context:
             filename = filename.replace(
-                '%service',
-                context['service'].module_name,
-            )
+                "%service", context["service"].module_name,)
 
         # Replace the %proto variable if appliable.
         # In the cases of protos, we also honor subpackages.
-        if context and 'proto' in context:
+        if context and "proto" in context:
             filename = filename.replace(
-                '%proto',
-                context['proto'].module_name,
-            )
+                "%proto", context["proto"].module_name,)
 
         # Paths may have empty path segments if components are empty
         # (e.g. no %version); handle this.
-        filename = re.sub(r'/+', '/', filename)
+        filename = re.sub(r"/+", "/", filename)
 
         # Done, return the filename.
         return filename
 
 
-__all__ = (
-    'Generator',
-)
+__all__ = ("Generator",)
