@@ -13,11 +13,9 @@
 # limitations under the License.
 
 from gapic.samplegen import samplegen
-from gapic.samplegen import utils
-from gapic.samplegen import yaml
+from gapic.samplegen import manifest
 
 __all__ = (
+    'manifest',
     'samplegen',
-    'utils',
-    'yaml',
 )
