@@ -832,6 +832,11 @@ class Service:
         """Return whether the service has a long-running method."""
         return any([m.lro for m in self.methods.values()])
 
+    @property
+    def has_pagers(self) -> bool:
+        """Return whether the service has paged methods."""
+        return any(m.paged_result_field for m in self.methods.values())
+
     @property
     def host(self) -> str:
         """Return the hostname for this service, if specified.
